% Demonstrates VSR encoding for place recognition. A run takes about 15-30
% minutes. This recreates one entry from the results table I from the
% paper.
%
function demo()
    basepath = '.';

    % load data    
    L1 = load(fullfile(basepath, 'data/2014-12-09-13-21-02/delf_pca.mat'), 'Y');
    S1 = load(fullfile(basepath, 'data/2014-12-09-13-21-02/pixel_classes_pred.mat'));
    
    L2 = load(fullfile(basepath, 'data/2015-05-19-14-06-38/delf_pca.mat'), 'Y');
    S2 = load(fullfile(basepath, 'data/2015-05-19-14-06-38/pixel_classes_pred.mat'));
    
    % setup parameters     
    params.nDims=4096; % number of descriptor dimensions
    params.nFeat=200; % number of appearance features
    params.relevantClasses = 1:11; % we only us non-dynamic entities
    params.minEntSize = 10; % skip tiny entities
    
    % create holistic descriptors D1 and D2 using HDC
    t1 = tic; D1 = encodeImageSetVSR(S1.C, L1.Y, params); t1=toc(t1);
    fprintf('encoding dataset 1 took %d seconds\n', t1); 
    %save('D1.mat', 'D1');
    
    t2 = tic; D2 = encodeImageSetVSR(S2.C, L2.Y, params); t2=toc(t2);
    fprintf('encoding dataset 2 took %d seconds\n', t2);
    %save('D2.mat', 'D2');
    
    % standardize and compare
    t3 = tic;
    S = simFHRR(D1, D2); 
    t3 = toc(t3);
    fprintf('comparing the VSR descriptors took %d seconds\n', t3);
    
    % load ground truth
    load(fullfile(basepath, 'data/gt.mat'), 'GT');
    
    % evaluate
    [P,R] = createPR(S, GT.GThard, GT.GTsoft);
    AUC = trapz(R,P)
    fprintf('The experiment refers to table I from the paper (VSR on first Oxford sequence) and above number should be 0.84\n');
    
    
end

% Compute a VSR for each image in L/Y
%
% L ... tensor of label images
% Y ... array of structs of local features
%
% D ... output descriptors, each row is a descriptor
%
function D = encodeImageSetVSR(L,Y,params)
    %nIm = min(params.nIms, numel(Y));
    nIm = numel(Y);
    D = zeros(nIm, params.nDims);
    for i=1:nIm
        if mod(i,100)==0, fprintf('running %d of %d\n', i, nIm); end
        D(i,:) = encodeImageVSR(L(:,:,i), Y(i), params);
    end
end