% This is the version that I want to use for the paper results
function main_encodeDatasetVSR
    
    params.minEntSize = 10;
    params.noHoodHeight = [800, 540];  %oxford, FreiburgME
    params.entityTmpPath = '/home/nepe/etit_arbeit/programs/pr_candidates/hdc/tmp/entitiesVSR/';
    params.delfPath = '/home/nepe/etit_arbeit/datasets/benchmark_journal/descriptors/';
    params.semSegPath = '/home/nepe/etit_arbeit/datasets/benchmark_journal/semantics/hss/';
    params.seedA = 102;
    params.relevantClasses = 1:11; % static cityscapes classe
    
    encID = 'VSR'; params.mode = 0;
%     encID = 'VSR_onlyF'; params.mode = 1;
%     encID = 'VSR_onlyCPH'; params.mode = 2;
    
     
%     dsNames = {'StLucia'};
%      dsNames = {'OxfordRobotCar', 'CMU' ,'StLucia'};
     dsNames = {'BonnME', 'FreiburgME'};
%      dsNames = {'GardensPointWalking','OxfordRobotCar', 'SFUMountain','CMU', 'Nordland1000','StLucia'};
    
    % load class statistics
%     load('hdc/descriptors/OxfordRobotCar/class_stat.mat', 'class_stat');
%     class_stat = class_stat(relevantClasses);
%     class_stat = class_stat/sum(class_stat);
    
    % prepare projection matrix
    rng(params.seedA);
    A = randn(1024, 4096, 'single');
    A = orth(A')';
    
    % process each dataset
    for dsIdx = 1:numel(dsNames)
        
        [E, ~] = getExperimentPaths(dsNames{dsIdx});
        
        % for each DB-Q
        for eIdx = 1:numel(E)
            fprintf('Dataset: %s\n', E(eIdx).gtPath);
            
            % ------ DB -------
            % generate savename             
            savename = [strrep(E(eIdx).dbPath, 'descriptors', 'descriptors_VSR') '/' encID '.mat'];
                        
            % if not exist, compute and store
            if ~exist(savename, 'file')
                SS = strsplit(E(eIdx).dbPath, '/');
                dsID = SS{end-1};
                seqID = SS{end};
                
                Y = computeVSR( dsID, seqID, A, params);
                
                savedir = strrep(E(eIdx).dbPath, 'descriptors', 'descriptors_VSR');
                if ~exist( savedir,'file' ); mkdir(savedir); end
                save(savename, 'Y', '-v7.3');
            end
                        
            % ------ Q -------
            % generate savename             
            savename = [strrep(E(eIdx).qPath, 'descriptors', 'descriptors_VSR') '/' encID '.mat'];
            
            % if not exist, compute and store
            if ~exist(savename, 'file')
                SS = strsplit(E(eIdx).qPath, '/');
                dsID = SS{end-1};
                seqID = SS{end};
                
                Y = computeVSR( dsID, seqID, A, params);
                
                savedir = strrep(E(eIdx).qPath, 'descriptors', 'descriptors_VSR');
                if ~exist( savedir,'file' ); mkdir(savedir); end
                save(savename, 'Y', '-v7.3');
            end
            
        end
    end
end

function D = computeVSR( dsID, seqID, A, params)
                
    % compute or load entities
    %entitiesPath = ['hdc/tmp/' seqID '_entities' nID];
    entitiesPath = fullfile(params.entityTmpPath, dsID, seqID);
    
    if exist(entitiesPath, 'file')
        fprintf('EXISTS %s\n', entitiesPath);
    else
        fprintf('creating %s\n', entitiesPath);
        mkdir(entitiesPath);
    end

    labelpath = fullfile(params.semSegPath, dsID, seqID, '/pixel_classes_pred.mat');    
    LL = matfile(labelpath); %, 'C', 'classes', 'numbers', 'colors'); 

    n = size(LL.C,3);

    nc = numel(params.relevantClasses);
    
    % process each image        
    for i=1:n

        savename = [entitiesPath sprintf('/entities%05d.mat', i)];
        if exist(savename, 'file')
            continue;
        end                

        if mod(i, 100)==0; fprintf('  %d of %d\n', i,n); end
        
        if ~isinf(params.noHoodHeight(1)) && strcmp(dsID, 'OxfordRobotCar')
            L = LL.C(1:params.noHoodHeight(1), :, i);
        elseif ~isinf(params.noHoodHeight(2)) && strcmp(dsID, 'FreiburgME')
            L = LL.C(1:params.noHoodHeight(2), :, i);
        else
            L = LL.C(:, :, i);
        end

        [E, classIdx] = getEntities(L, params.relevantClasses, params);

        entity.E = E;
        entity.classIDx = classIdx;

        save(savename, 'E', 'classIdx');

    end

    % load DELF
    DELF = load(fullfile(params.delfPath, dsID, seqID, '/delf.mat')); 

    % encode
    % for each image
    %n = min(nMax, numel(entities));
    D = zeros(n,4096);
    for i=1:n
        if mod(i, 100)==0; fprintf('  %d of %d\n', i,n); end

        % load entities
        loadname = [entitiesPath sprintf('/entities%05d.mat', i)];
        load(loadname, 'E');

        Di = encodeEntities(E, nc, DELF.Y(i), A, params.mode);
        if ~isempty(Di)
            D(i,:) = Di;
        else
            warning('zero image descriptor at image %d', i);
        end


    end
end

